@@ -49,8 +49,12 @@ def bash_command_string() -> str:
     return '/usr/bin/env bash -o pipefail'
 
 
-def system_statistics_collection_period() -> int:
-    """How often should system statistics be collected in seconds"""
+def system_statistics_collection_period() -> typing.Union[float, None]:
+    """
+    How often should system statistics be collected in seconds.
+
+    When zero or None the collecting of system statistics is disabled.
+    """
     return 1
 
 
@@ -64,6 +68,11 @@ def allow_run_from_web_ui() -> bool:
     return True
 
 
+def display_system_statistics() -> bool:
+    """If the system statistics shall be visible in the web UI."""
+    return True
+
+
 def base_url() -> str:
     """External url of flask app, for linking nodes in slack messages"""
     return 'http://127.0.0.1:5000/pipelines'
