@@ -14,6 +14,8 @@ import atexit
 import time
 import traceback
 from multiprocessing import queues
+from multiprocessing.context import BaseContext
+from queue import Empty
 
 from . import pipelines, config
 from .logging import logger, pipeline_events, system_statistics, run_log, node_cost
@@ -53,9 +55,18 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
     # A queue for receiving events from forked sub processes
     event_queue = multiprocessing_context.Queue()
 
+    # The historical node cost is taken from the 'mara' db. If the 'mara' datatabase is not defined
+    # (e.g. in testing scenarios) we do not use the node cost.
+    import mara_db.config
+    use_historical_node_cost = 'mara' in mara_db.config.databases()
+    if not use_historical_node_cost:
+        print(f"[WARNING] The 'mara' database is not defined. The historical node costs are not used, the execution path might be inefficient.", file=sys.stderr)
+
     # The function that is run in a sub process
     def run():
 
+        statistics_process: multiprocessing.Process = None
+
         try:
             # capture output of print statements and other unplanned output
             logger.redirect_output(event_queue, pipeline.path())
@@ -64,7 +75,7 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
             node_queue: [pipelines.Node] = []
 
             # data needed for computing cost
-            node_durations_and_run_times = node_cost.node_durations_and_run_times(pipeline)
+            node_durations_and_run_times = node_cost.node_durations_and_run_times(pipeline) if use_historical_node_cost else {}
 
             # Putting nodes into the node queue
             def queue(nodes: [pipelines.Node]):
@@ -116,7 +127,8 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
                         if tp.is_alive():
                             # give it a chance to gracefully shutdown
                             tp.terminate()
-                    statistics_process.kill()
+                    if statistics_process:
+                        statistics_process.kill()
                 except BaseException as e:
                     print(f"Exception during TaskProcess cleanup: {repr(e)}", file=sys.stderr, flush=True)
                 return
@@ -179,9 +191,10 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
                             )
 
             # collect system stats in a separate Process
-            statistics_process = multiprocessing.Process(
-                target=lambda: system_statistics.generate_system_statistics(event_queue), name='system_statistics')
-            statistics_process.start()
+            if config.system_statistics_collection_period():
+                statistics_process = multiprocessing.Process(
+                    target=lambda: system_statistics.generate_system_statistics(event_queue), name='system_statistics')
+                statistics_process.start()
 
             # run as long
             # - as task processes are still running
@@ -207,7 +220,8 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
                                         next_node.add_dependency(pipeline_node, downstream)
 
                             # get cost information for children
-                            node_durations_and_run_times.update(node_cost.node_durations_and_run_times(next_node))
+                            if use_historical_node_cost:
+                                node_durations_and_run_times.update(node_cost.node_durations_and_run_times(next_node))
 
                             # queue all child nodes
                             queue(list(next_node.nodes.values()))
@@ -258,8 +272,7 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
                                 message='★ ' + node_cost.format_duration(
                                     node_durations_and_run_times.get(tuple(next_node.path()), [0, 0])[0])))
 
-                            status_queue = multiprocessing_context.Queue()
-                            process = TaskProcess(next_node, event_queue, status_queue)
+                            process = TaskProcess(next_node, event_queue, multiprocessing_context)
                             process.start()
                             running_task_processes[next_node] = process
 
@@ -274,19 +287,18 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
 
                         processed_nodes.add(task_process.task)
 
-                        succeeded = not (task_process.status_queue.get() == False or task_process.exitcode != 0)
-                        if not succeeded and not task_process.task.parent.ignore_errors:
+                        if not task_process.succeeded and not task_process.task.parent.ignore_errors:
                             for parent in task_process.task.parents()[:-1]:
                                 failed_pipelines.add(parent)
 
                         end_time = datetime.datetime.now(tz.utc)
                         event_queue.put(
                             pipeline_events.Output(task_process.task.path(),
-                                                   ('succeeded' if succeeded else 'failed') + ',  '
+                                                   ('succeeded' if task_process.succeeded else 'failed') + ',  '
                                                    + logger.format_time_difference(task_process.start_time, end_time),
-                                                   format=logger.Format.ITALICS, is_error=not succeeded))
+                                                   format=logger.Format.ITALICS, is_error=not task_process.succeeded))
                         event_queue.put(pipeline_events.NodeFinished(task_process.task.path(), task_process.start_time,
-                                                                     end_time, False, succeeded))
+                                                                     end_time, False, task_process.succeeded))
 
                 # check if some pipelines finished
                 track_finished_pipelines()
@@ -301,9 +313,10 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
         # run again because `dequeue` might have moved more nodes to `finished_nodes`
         track_finished_pipelines()
 
-        # kill the stats process (joining or terminating does not work in gunicorn)
-        os.kill(statistics_process.pid, signal.SIGKILL)
-        statistics_process.join()
+        if statistics_process:
+            # kill the stats process (joining or terminating does not work in gunicorn)
+            os.kill(statistics_process.pid, signal.SIGKILL)
+            statistics_process.join()
 
         # run finished
         event_queue.put(pipeline_events.RunFinished(node_path=pipeline.path(), end_time=datetime.datetime.now(tz.utc),
@@ -314,7 +327,10 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
     run_process = multiprocessing_context.Process(target=run, name='pipeline-' + '-'.join(pipeline.path()))
     run_process.start()
 
-    runlogger = run_log.RunLogger()
+    runlogger = initialize_run_logger()
+    if not runlogger:
+        runlogger = run_log.NullLogger()
+        print(f"[WARNING] The events of the pipeline execution are not saved in a db", file=sys.stderr)
 
     # make sure that we close this run (if still open) as failed when we close this python process
     # On SIGKILL we will still leave behind open runs...
@@ -394,21 +410,38 @@ def run_pipeline(pipeline: pipelines.Pipeline, nodes: {pipelines.Node} = None,
         time.sleep(0.001)
 
 
-class TaskProcess(multiprocessing.Process):
-    def __init__(self, task: pipelines.Task, event_queue: multiprocessing.Queue, status_queue: multiprocessing.Queue):
+def initialize_run_logger() -> events.EventHandler:
+    """
+    Initializes the base run logger used during pipeline execution.
+
+    Note:
+        The event handler must have an attribute 'run_id' which holds a
+        unique id for the current execution after event RunStarted is processed.
+    """
+    import mara_db.config
+    if 'mara' in mara_db.config.databases():
+        return run_log.RunLogger()
+
+
+class TaskProcess:
+    def __init__(self, task: pipelines.Task, event_queue: multiprocessing.Queue, multiprocessing_context: BaseContext):
         """
         Runs a task in a separate sub process.
 
         Args:
             task: The task to run
             event_queue: The query for writing events to
-            status_queue: A queue for reporting whether the task succeeded
+            multiprocessing_context: The multiprocessing context in which the task process shall run
         """
-        super().__init__(name='task-' + '-'.join(task.path()))
+        self._process: multiprocessing.Process = multiprocessing_context.Process(
+            name='task-' + '-'.join(task.path()),
+            target=TaskProcess.run,
+            args=(self,))
         self.task = task
         self.event_queue = event_queue
-        self.status_queue = status_queue
+        self._status_queue = multiprocessing_context.Queue()
         self.start_time = datetime.datetime.now(tz.utc)
+        self._succeeded: bool = None
 
     def run(self):
         # redirect stdout and stderr to queue
@@ -435,4 +468,35 @@ class TaskProcess(multiprocessing.Process):
             logger.log(message=traceback.format_exc(), format=logger.Format.VERBATIM, is_error=True)
             succeeded = False
 
-        self.status_queue.put(succeeded)
+        self._status_queue.put(succeeded)
+
+    def start(self):
+        self._process.start()
+
+    def terimate(self):
+        self._process.terminate()
+
+    def kill(self):
+        self._process.kill()
+
+    def join(self, timeout=None):
+        self._process.join(timeout=timeout)
+
+    def is_alive(self):
+        return self._process.is_alive()
+
+    @property
+    def succeeded(self):
+        if self._succeeded is None:
+            if self.is_alive():
+                return None
+
+            succeeded_from_queue = None
+            try:
+                succeeded_from_queue = self._status_queue.get(False)
+            except Empty:
+                pass
+
+            self._succeeded = succeeded_from_queue == True
+
+        return self._succeeded
